#ifndef MAT_HEAD
#define MAT_HEAD

#include <math.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_complex.h>
#include <gsl/gsl_complex_math.h>

class zespolone;
char* nazwafunkcji(int);
bool czygorna(int);

class zespolone{
private:
gsl_complex zesp;  
  public:
    // rzutowanie zespolone na gsl_complex
    // powinno umo?liwi? odpalanie bibliotek gsl_complex na zespolone.
  operator gsl_complex() const {return zesp;}
  // operator rzutowania typu
  zespolone(gsl_complex z) {zesp=z;}
  // obserwatory
  double re() const {return GSL_REAL(zesp);}
  double im() const {return GSL_IMAG(zesp);}
  // konstruktory: domy?lny, rzeczywisty i zespolony.
  zespolone() {zesp=gsl_complex_rect(0.0,0.0);}
  zespolone(double x) {zesp=gsl_complex_rect(x,0.0);}
  zespolone(double x,double y) {zesp=gsl_complex_rect(x,y);}
  // pewne wa?ne funkcje
  zespolone odwrot() const {return gsl_complex_inverse(zesp);}
  double arg() const {return gsl_complex_arg(zesp);}
  double modul() const {return gsl_complex_abs(zesp);}
  double modul2() const {return gsl_complex_abs2(zesp);}
  // podstawowe operatory matemayczne.
  friend const zespolone operator+(const zespolone& z1, const zespolone& z2) 
  { return gsl_complex_add(z1,z2);}
  friend const zespolone operator*(const zespolone& z1, const zespolone& z2)
  {return gsl_complex_mul(z1,z2);}
  friend const zespolone operator/(const zespolone& z1, const zespolone& z2)
  {return gsl_complex_div(z1,z2);}
  friend const zespolone operator-(const zespolone& z1, const zespolone& z2)
  {return gsl_complex_sub(z1,z2);}
  // testy rwno?ci
  friend const bool operator==(const zespolone& z1, const zespolone& z2);
  friend const bool operator!=(const zespolone& z1,const zespolone& z2);
  // testy zerowania
  const bool isZero() const {return (re()==0.0 && im()==0.0);}
  const bool inNotZero() const {return (!isZero());}
      
  
  // umo?liwia odpalanie elementow
  /*
  long double re() {return rex;}
  long double im() {return imx;}
  long double modul2() {return rex*rex+imx*imx;}
  zespolone() {rex=0; imx=0;}
  zespolone(long double x, long double y) {rex=x; imx=y;}
  zespolone operator*(zespolone z) {return zespolone(rex*z.rex-imx*z.imx,rex*z.imx+imx*z.rex);}
  zespolone odwrot() {long double m=rex*rex+imx*imx; return zespolone(rex/m,-imx/m);}
  zespolone operator+(zespolone z) {return zespolone(rex+z.rex,imx+z.imx);}
  zespolone operator+(long double d) {return zespolone(rex+d,imx);}
  zespolone operator*(long double m) {return zespolone(rex*m,imx*m);}
  zespolone operator-(zespolone z) {return zespolone(rex-z.rex,imx-z.imx);}
  zespolone operator-(long double d) {return zespolone(rex-d,imx);}
  bool operator==(zespolone z) {return ((rex==z.rex)&&(imx==z.imx));}
  bool operator==(long double d) {return ((rex==d) && (imx==0));}
  bool isNonPositiveInteger() {return ((rex<=0) && (imx==0) && (rex==int(rex)));}
  long double modul() {return sqrtl(modul2());}
      */
  friend bool homografia(zespolone& z,zespolone a,zespolone b,zespolone c,zespolone d);
  friend bool zukowski(zespolone& z);
  friend bool expz(zespolone& z);
  friend bool kwadrat(zespolone& z,zespolone a,zespolone b);
  friend bool szescian(zespolone& z,zespolone a,zespolone b, zespolone c);
  friend bool digamma(zespolone& z);
  friend bool przeksztalcenie(zespolone& z,zespolone a,zespolone b, zespolone c, zespolone d,int jakie);
  friend bool arcsine(zespolone& z);
  friend bool logarithm(zespolone& z);
  friend bool squareroot(zespolone& z);
  friend bool tangens(zespolone& z);
  friend bool arctangens(zespolone& z);
  friend bool arctangensh(zespolone& z);
  friend bool bezcwiartki(zespolone& z);
  friend bool SC2(zespolone& z);
  friend bool logcoth(zespolone& z);
  friend bool SC3(zespolone& z);
  friend bool SC4(zespolone& z);
};
#endif

zespolone liczbazarg(double r, double phi);
const zespolone potega(const zespolone& z,const zespolone& w);
